using System;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Authentication;
using System.Text;

namespace Vevo.Shared.Utilities
{
    public class HashProcessor
    {
        #region private

        private const int SaltSizeInBytes = 16;

        private HashAlgorithmType _hashAlgorithmType;


        private byte[] ConcatenateBytes( byte[] bytes1, byte[] bytes2 )
        {
            byte[] result = new byte[bytes1.Length + bytes2.Length];
            Buffer.BlockCopy( bytes1, 0, result, 0, bytes1.Length );
            Buffer.BlockCopy( bytes2, 0, result, bytes1.Length, bytes2.Length );

            return result;
        }

        private void SeparateBytes( byte[] source, int indexToSeparate, out byte[] out1, out byte[] out2 )
        {
            int length1, length2;
            if (indexToSeparate <= source.Length)
            {
                length1 = indexToSeparate;
                length2 = source.Length - indexToSeparate;
            }
            else
            {
                length1 = source.Length;
                length2 = 0;
            }

            out1 = new byte[length1];
            out2 = new byte[length2];

            Buffer.BlockCopy( source, 0, out1, 0, out1.Length );
            Buffer.BlockCopy( source, indexToSeparate, out2, 0, out2.Length );
        }

        private byte[] GenerateSalt()
        {
            byte[] buf = new byte[SaltSizeInBytes];
            (new RNGCryptoServiceProvider()).GetBytes( buf );
            return buf;
        }

        private byte[] EncodeToBytes( string plainText, byte[] bSalt )
        {
            byte[] bIn = Encoding.Unicode.GetBytes( plainText );

            byte[] bAll = ConcatenateBytes( bSalt, bIn );

            HashAlgorithm hashAlgorithm = HashAlgorithm.Create( _hashAlgorithmType.ToString() );
            return hashAlgorithm.ComputeHash( bAll );
        }

        #endregion


        #region Constructors

        public HashProcessor( HashAlgorithmType hashAlgorithmType )
        {
            _hashAlgorithmType = hashAlgorithmType;
        }

        #endregion


        #region Public Methods

        public string EncodePassword( string password, out string passwordSalt )
        {
            byte[] saltBytes = GenerateSalt();
            passwordSalt = Convert.ToBase64String( saltBytes );

            byte[] encoded = EncodeToBytes( password, saltBytes );
            return Convert.ToBase64String( encoded );
        }

        public string EncodePassword( string password, string passwordSalt )
        {
            byte[] saltBytes = Convert.FromBase64String( passwordSalt );

            byte[] encoded = EncodeToBytes( password, saltBytes );
            return Convert.ToBase64String( encoded );
        }

        public string EncodeWithRandomSalt( string plainText )
        {
            byte[] saltBytes = GenerateSalt();
            byte[] encoded = EncodeToBytes( plainText, saltBytes );

            byte[] result = ConcatenateBytes( saltBytes, encoded );
            return Convert.ToBase64String( result );
        }

        public bool VerifyHash( string plainText, string hashWithSalt )
        {
            byte[] hashWithSaltBytes = Convert.FromBase64String( hashWithSalt );

            if (hashWithSaltBytes.Length < SaltSizeInBytes)
                return false;

            byte[] saltBytes, encoded;
            SeparateBytes( hashWithSaltBytes, SaltSizeInBytes, out saltBytes, out encoded );

            byte[] newHash = EncodeToBytes( plainText, saltBytes );

            return Convert.ToBase64String( newHash ) == Convert.ToBase64String( encoded );
        }

        #endregion
    }
}
