using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Contents;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ContentMenuItemRepository : IContentMenuItemRepository
    {
        #region Private
        private const string RepositoryName = "ContentMenuItemRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetLocalesSignature = "GetLocales";
        private const string SearchContentMenuItemSignature = "SearchContentMenuItem";
        private const string GetByContentMenuIDSignature = "GetByContentMenuID";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetAllSignature = "GetAll";
        private const string GetContentMenuItemIDFromUrlNameSignature = "GetContentMenuItemIDFromUrlName";
        private const string GetByPositionSignature = "GetByPosition";
        private const string GetByStoreIDSignature = "GetByStoreID";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "ContentMenuItemLocale", "ContentMenuItem" } );

        private ContentMenuItemRepositoryNoCache _repository = new ContentMenuItemRepositoryNoCache();

        #endregion

        #region Constructors

        public ContentMenuItemRepository()
        {
        }

        public ContentMenuItemRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public ContentMenuItem Save( ContentMenuItem contentmenuitem )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            ContentMenuItem newContentMenuItem = _repository.Save( contentmenuitem );
            _dataCache.ClearRepositoryCache();
            return newContentMenuItem;
        }

        public ContentMenuItem GetOne( Culture culture, string contentmenuitemID )
        {
            ContentMenuItem result = (ContentMenuItem) _dataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, contentmenuitemID );

            if (result == null)
            {
                result = _repository.GetOne( culture, contentmenuitemID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, contentmenuitemID );
            }

            return result;
        }

        public IList<ContentMenuItem> GetByContentMenuID( Culture culture, string contentMenuID, string sortBy, BoolFilter isEnabled )
        {
            IList<ContentMenuItem> list = (IList<ContentMenuItem>) _dataCache.GetValue(
                GetByContentMenuIDSignature, _cacheDependency, culture.CultureID, contentMenuID, sortBy, isEnabled );

            if (list == null)
            {
                list = _repository.GetByContentMenuID( culture, contentMenuID, sortBy, isEnabled );
                _dataCache.SetValue( GetByContentMenuIDSignature, list, culture.CultureID, contentMenuID, sortBy, isEnabled );
            }

            return list;
        }

        public IList<ContentMenuItem> GetByContentMenuID( Culture culture, string contentMenuID, string sortBy, BoolFilter isEnabled, BoolFilter contentIsEnabled )
        {
            IList<ContentMenuItem> list = (IList<ContentMenuItem>) _dataCache.GetValue(
                GetByContentMenuIDSignature, _cacheDependency, culture.CultureID, contentMenuID, sortBy, isEnabled, contentIsEnabled );

            if (list == null)
            {
                list = _repository.GetByContentMenuID( culture, contentMenuID, sortBy, isEnabled, contentIsEnabled );
                _dataCache.SetValue( GetByContentMenuIDSignature, list, culture.CultureID, contentMenuID, sortBy, isEnabled, contentIsEnabled );
            }

            return list;
        }

        public IList<ContentMenuItem> GetByPosition( Culture culture, string menuPosition, string sortBy, BoolFilter isEnabled )
        {
            IList<ContentMenuItem> list = (IList<ContentMenuItem>) _dataCache.GetValue(
                GetByPositionSignature, _cacheDependency, culture.CultureID, menuPosition, sortBy, isEnabled );

            if (list == null)
            {
                list = _repository.GetByPosition( culture, menuPosition, sortBy, isEnabled );
                _dataCache.SetValue( GetByPositionSignature, list, culture.CultureID, menuPosition, sortBy, isEnabled );
            }

            return list;
        }

        public string GetContentMenuItemIDFromUrlName( string urlName )
        {
            object contentMenuItemID = _dataCache.GetValue( GetContentMenuItemIDFromUrlNameSignature, _cacheDependency, urlName );

            if (contentMenuItemID == null)
            {
                contentMenuItemID = _repository.GetContentMenuItemIDFromUrlName( urlName );
                _dataCache.SetValue( GetContentMenuItemIDFromUrlNameSignature, contentMenuItemID, urlName );
            }

            return contentMenuItemID.ToString();
        }

        public void Delete( string contentmenuitemID )
        {
            _repository.Delete( contentmenuitemID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string contentmenuitemID )
        {
            _repository.DeleteLocalesByCultureID( contentmenuitemID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<ILocale> GetLocales( string contentmenuitemID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, contentmenuitemID );

            if (list == null)
            {
                list = _repository.GetLocales( contentmenuitemID );
                _dataCache.SetValue( GetLocalesSignature, list, contentmenuitemID );
            }

            return list;
        }

        public IList<ContentMenuItem> GetAll( Culture culture, BoolFilter enabled )
        {
            IList<ContentMenuItem> list = (IList<ContentMenuItem>) _dataCache.GetValue(
                GetAllSignature,
                _cacheDependency,
                culture.CultureID,
                enabled.ToString() );

            if (list == null)
            {
                list = _repository.GetAll( culture, enabled );
                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, enabled.ToString() );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }

            return list;
        }

        public IList<ContentMenuItem> SearchContentMenuItem(
            Culture culture,
            string menuPosition,
            string sortBy,
            SearchFilter searchFilter )
        {
            Pair result = (Pair) _dataCache.GetValue(
                SearchContentMenuItemSignature,
                _cacheDependency,
                culture.CultureID,
                menuPosition,
                sortBy,
                searchFilter.ToString() );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchContentMenuItem(
                    culture, menuPosition, sortBy, searchFilter );
                _dataCache.SetValue(
                    SearchContentMenuItemSignature,
                    result,
                    culture.CultureID,
                    menuPosition,
                    sortBy,
                    searchFilter.ToString() );
            }

            return (IList<ContentMenuItem>) result.First;
        }


        // This function always read from database, not from cache, since maximum sort order may change.

        public string GetMaxSortOrder( string contentMenuID )
        {
            return _repository.GetMaxSortOrder( contentMenuID );
        }

        public void ReArrangeSortOrderByContentMenuID( Culture culture, string parentContentMenuID,
           BoolFilter enabled )
        {
            _repository.ReArrangeSortOrderByContentMenuID( culture, parentContentMenuID,
                enabled );
            _dataCache.ClearRepositoryCache();
        }

        public void ReArrangeSortOrder( Culture culture, string parentContentMenuID,
            string contentMenuItemID, BoolFilter enabled )
        {
            _repository.ReArrangeSortOrder( culture, parentContentMenuID, contentMenuItemID,
                enabled );
            _dataCache.ClearRepositoryCache();
        }

        public void UpdateSortOrder( string[] sortList )
        {
            _repository.UpdateSortOrder( sortList );
            _dataCache.ClearRepositoryCache();
        }

        public IList<ContentMenuItem> GetByStoreID( Culture culture, string storeID, string sortBy, BoolFilter isEnabled )
        {
            IList<ContentMenuItem> list = (IList<ContentMenuItem>) _dataCache.GetValue(
                GetByContentMenuIDSignature, _cacheDependency, culture.CultureID, storeID, sortBy, isEnabled );

            if (list == null)
            {
                list = _repository.GetByStoreID( culture, storeID, sortBy, isEnabled );
                _dataCache.SetValue( GetByStoreIDSignature, list, culture.CultureID, storeID, sortBy, isEnabled );
            }

            return list;
        }

        #endregion

    }
}
