using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;

using Vevo.Data.Implementation;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Discounts;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class DiscountGroupRepositoryNoCache : IDiscountGroupRepository
    {
        #region Private

        private DiscountRuleAccess _discountRuleAccess = new DiscountRuleAccess();

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private DiscountGroup Create( DiscountGroup discountGroup )
        {
            discountGroup.DiscountGroupID = DataAccess.ExecuteScalar(
                "INSERT INTO DiscountGroup ( GroupName, DiscountType, ProductOptionDiscount ) " +
                "VALUES ( @GroupName, @DiscountType, @ProductOptionDiscount );" +
                "SELECT @@Identity;",
                DataAccess.CreateParameterString( discountGroup.GroupName ),
                DataAccess.CreateParameterString( discountGroup.DiscountType.ToString() ),
                DataAccess.CreateParameterString( discountGroup.ProductOptionDiscount.ToString() ) );

            return discountGroup;
        }

        private DiscountGroup Update( DiscountGroup discountGroup )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE DiscountGroup " +
                "SET GroupName = @GroupName, " +
                "DiscountType = @DiscountType, " +
                "ProductOptionDiscount = @ProductOptionDiscount " +
                "WHERE DiscountGroupID = @DiscountGroupID;",
                DataAccess.CreateParameterString( discountGroup.GroupName ),
                DataAccess.CreateParameterString( discountGroup.DiscountType.ToString() ),
                DataAccess.CreateParameterString( discountGroup.ProductOptionDiscount.ToString() ),
                DataAccess.CreateParameterString( discountGroup.DiscountGroupID ) );

            return discountGroup;
        }

        private DiscountGroup CreateOneDiscountGroupFromRow( DataRow row )
        {
            DiscountGroup discountGroup = new DiscountGroup();
            discountGroup.DiscountGroupID = row["DiscountGroupID"].ToString();
            discountGroup.GroupName = row["GroupName"].ToString();
            discountGroup.DiscountType =
                EnumUtilities<DiscountGroup.DiscountTypeEnum>.Parse( row["DiscountType"].ToString() );
            discountGroup.ProductOptionDiscount = ConvertUtilities.ToBoolean( row["ProductOptionDiscount"] );

            return discountGroup;
        }

        private IList<DiscountGroup> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<DiscountGroup> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<DiscountGroup> list = new List<DiscountGroup>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneDiscountGroupFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion


        // Functions that are intended to be used internally inside aggregates.
        #region Aggregate Methods

        public IList<DiscountRule> GetDiscountRulesByDiscountGroupID( string discountGroupID )
        {
            return _discountRuleAccess.GetByDiscountGroupID( discountGroupID );
        }

        #endregion


        #region Public Methods

        public DiscountGroup Save( DiscountGroup discountGroup )
        {
            DiscountGroup result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (ConvertUtilities.ToInt32( discountGroup.DiscountGroupID ) == 0)
                    result = Create( discountGroup );
                else
                    result = Update( discountGroup );

                _discountRuleAccess.SaveAll(
                    discountGroup.DiscountGroupID,
                    discountGroup.DiscountRules );

                transaction.Complete();
            }

            return result;
        }

        public DiscountGroup GetOne( string discountGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * FROM DiscountGroup " +
                "WHERE DiscountGroupID = @DiscountGroupID;",
                DataAccess.CreateParameterString( discountGroupID ) );

            if (table.Rows.Count > 0)
                return CreateOneDiscountGroupFromRow( table.Rows[0] );
            else
                return DiscountGroup.Null;
        }

        public void Delete( string discountGroupID )
        {
            DiscountGroup discountGroup = GetOne( discountGroupID );
            discountGroup.DiscountRules.Clear();

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                "DELETE FROM DiscountGroup " +
                "WHERE DiscountGroupID = @DiscountGroupID;",
                DataAccess.CreateParameterString( discountGroupID ) );

                _discountRuleAccess.SaveAll(
                    discountGroup.DiscountGroupID,
                    discountGroup.DiscountRules );

                transaction.Complete();
            }
        }

        public IList<DiscountGroup> GetAll()
        {
            DataTable table = DataAccess.ExecuteSelect( "SELECT * FROM DiscountGroup" );
            return ConvertTableToList( table );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * FROM  DiscountGroup " +
                "WHERE 1 = 0 " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<DiscountGroup> SearchDiscountGroup(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT * " +
                "FROM DiscountGroup  " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["DiscountGroup.DiscountGroupID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( full, startIndex, endIndex );
        }

        #endregion
    }
}
