using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Payments;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Data.Implementation;

namespace Vevo.Data
{
    public class PaymentOptionRepositoryNoCache : IPaymentOptionRepository
    {
        #region Private

        private const string AllPaymentOptionFields = " PaymentOption.Name, DisplayName, Description, PaymentImage, " +
            "IsEnabled, Currencies, ShowPayment, ShowButton, ButtonUserControl, AdminUserControl, SortOrder, " +
            "ProviderClassName, SystemUseOnly, AuthenticationConfigs, CanUseRecurring, IsCreditCardNumberSaved, " +
            "SupportedCreditCards, SupportedCreditCardValues, PaymentMethodSelectionAllowed, AnonymousCheckoutAllowed, " +
            "BillingAddressRequired, CreditCardRequired, ShownAsCreditCard, Cvv2Required, IntegratedProviderClassName ";

        private const string AllPaymentOptionLocaleFields = " Name, CultureID, DisplayName, Description ";

        private PaymentOptionStoreAccess _paymentOptionStoreAccess = new PaymentOptionStoreAccess();

        private void SetPrimaryKey( DataTable table )
        {
            DataColumn[] keys = new DataColumn[1] { table.Columns["Name"] };
            table.PrimaryKey = keys;
        }

        private PaymentOptionLocale CreateOneLocaleFromRow( DataRow row )
        {
            PaymentOptionLocale locale = new PaymentOptionLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.DisplayName = row["DisplayName"].ToString();
            locale.Description = row["Description"].ToString();

            return locale;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private void LocalesSaveAll( string paymentName, LocaleCollection<PaymentOptionLocale> paymentOptionLocales )
        {
            DeleteLocales( paymentName );
            InsertLocales( paymentName, paymentOptionLocales );
        }

        private void DeleteLocales( string name )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM PaymentOptionLocale " +
                "WHERE Name = @Name; ",
                DataAccess.CreateParameterString( name ) );
        }

        private void InsertLocales( string paymentName, LocaleCollection<PaymentOptionLocale> paymentOptionLocales )
        {
            foreach (PaymentOptionLocale locale in paymentOptionLocales)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO PaymentOptionLocale " +
                    "( Name, CultureID, DisplayName, Description ) " +
                    "VALUES ( @Name, @CultureID, @DisplayName, @Description ) ",
                    DataAccess.CreateParameterString( paymentName ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.DisplayName ),
                    DataAccess.CreateParameterString( locale.Description ) );
            }
        }

        private PaymentOption UpdatePaymentOption( PaymentOption paymentOption )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE PaymentOption " +
                "SET PaymentImage = @PaymentImage, " +
                "SortOrder = @SortOrder, " +
                "IsEnabled = @IsEnabled, " +
                "Cvv2Required = @Cvv2Required " +
                "WHERE Name = @Name; ",
                DataAccess.CreateParameterString( paymentOption.PaymentImage ),
                DataAccess.CreateParameterInt32( paymentOption.SortOrder ),
                DataAccess.CreateParameterBool( paymentOption.IsEnabled ),
                DataAccess.CreateParameterBool( paymentOption.Cvv2Required ),
                DataAccess.CreateParameterString( paymentOption.Name ) );

            LocalesSaveAll( paymentOption.Name, paymentOption.Locales );
            
            return paymentOption;
        }

        private PaymentOption CreateOneFromRow( Culture culture, DataRow row )
        {
            PaymentOption paymentOption = new PaymentOption( culture );

            paymentOption.IntializeReadOnlyProperties(
                ConvertUtilities.ToString( row["Name"] ),
                StringUtilities.SplitString( row["Currencies"].ToString(), ',' ),
                ConvertUtilities.ToBoolean( row["ShowPayment"] ),
                ConvertUtilities.ToBoolean( row["ShowButton"] ),
                ConvertUtilities.ToString( row["ButtonUserControl"] ),
                ConvertUtilities.ToString( row["AdminUserControl"] ),
                ConvertUtilities.ToString( row["ProviderClassName"] ),
                ConvertUtilities.ToString( row["IntegratedProviderClassName"] ),
                ConvertUtilities.ToBoolean( row["SystemUseOnly"] ),
                ConvertUtilities.ToString( row["AuthenticationConfigs"] ),
                ConvertUtilities.ToBoolean( row["CanUseRecurring"] ),
                ConvertUtilities.ToBoolean( row["IsCreditCardNumberSaved"] ),
                ConvertUtilities.ToString( row["SupportedCreditCards"] ),
                ConvertUtilities.ToString( row["SupportedCreditCardValues"] ),
                ConvertUtilities.ToBoolean( row["PaymentMethodSelectionAllowed"] ),
                ConvertUtilities.ToBoolean( row["AnonymousCheckoutAllowed"] ),
                ConvertUtilities.ToBoolean( row["BillingAddressRequired"] ),
                ConvertUtilities.ToBoolean( row["CreditCardRequired"] ),
                ConvertUtilities.ToBoolean( row["ShownAsCreditCard"] ) );

            paymentOption.PaymentImage = ConvertUtilities.ToString( row["PaymentImage"] );
            paymentOption.IsEnabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );
            paymentOption.SortOrder = ConvertUtilities.ToInt32( row["SortOrder"] );
            paymentOption.Cvv2Required = ConvertUtilities.ToBoolean( row["Cvv2Required"] );
            paymentOption.Description = ConvertUtilities.ToString( row["Description"] );
            paymentOption.DisplayName = ConvertUtilities.ToString( row["DisplayName"] );


            return paymentOption;
        }

        private IList<PaymentOption> ConvertTableToList( Culture culture, DataTable table )
        {
            IList<PaymentOption> list = new List<PaymentOption>();

            for (int i = 0; i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }

        private IList<PaymentOption> GetPaymentOptionList(
            Culture culture,
            BoolFilter isEnabled,
            BoolFilter showPayment,
            BoolFilter showButton )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllPaymentOptionFields +
                "FROM PaymentOption LEFT JOIN " +
                "(SELECT * FROM PaymentOptionLocale WHERE CultureID = @CultureID) AS L " +
                "ON PaymentOption.Name = L.Name " +
                "WHERE SystemUseOnly = @SystemUseOnly " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                FlagFilterHelper.CreateFlagFilterWithAnd( "ShowPayment", RepositoryUtilities.ConvertToFlagFilter( showPayment ) ) +
                FlagFilterHelper.CreateFlagFilterWithAnd( "ShowButton", RepositoryUtilities.ConvertToFlagFilter( showButton ) ) +
                "ORDER BY SortOrder; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterBool( false ) );

            SetPrimaryKey( table );
            return ConvertTableToList( culture, table );
        }

        private void CopyPaymentOptionLocales( PaymentOption paymentOption, IList<ILocale> locales )
        {
            foreach (PaymentOptionLocale locale in locales)
            {
                paymentOption.Locales.Add( locale );
            }
        }

        #endregion


        #region Public Methods

        public PaymentOption Update( PaymentOption paymentOption )
        {
            PaymentOption result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                result = UpdatePaymentOption( paymentOption );
                transaction.Complete();
            }

            return result;
        }

        public PaymentOption GetOne( Culture culture, string name )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllPaymentOptionFields +
                "FROM PaymentOption LEFT JOIN " +
                " (SELECT * FROM PaymentOptionLocale WHERE CultureID = @CultureID) AS L " +
                "ON PaymentOption.Name = L.Name " +
                "WHERE PaymentOption.Name = @Name; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( name ) );

            if (table.Rows.Count > 0)
            {
                PaymentOption paymentOption = CreateOneFromRow( culture, table.Rows[0] );
                CopyPaymentOptionLocales( paymentOption, GetLocales( name ) );
                return paymentOption;
            }
            else
                return PaymentOption.Null;
        }

        public IList<ILocale> GetLocales( string name )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllPaymentOptionLocaleFields +
                "FROM PaymentOptionLocale " +
                "WHERE Name = @Name; ",
                DataAccess.CreateParameterString( name ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<PaymentOption> GetAll( Culture culture, BoolFilter isEnabled )
        {
            return GetPaymentOptionList( culture, isEnabled, BoolFilter.ShowAll, BoolFilter.ShowAll );
        }

        public IList<PaymentOption> GetShownPaymentList( Culture culture, BoolFilter isEnabled )
        {
            return GetPaymentOptionList( culture, isEnabled, BoolFilter.ShowTrue, BoolFilter.ShowAll );
        }

        public IList<PaymentOption> GetButtonList( Culture culture, BoolFilter isEnabled )
        {
            return GetPaymentOptionList( culture, isEnabled, BoolFilter.ShowAll, BoolFilter.ShowTrue );
        }

        public PaymentOptionStore CreatePaymentOptionStore( PaymentOptionStore poStore )
        {
            return _paymentOptionStoreAccess.Create( poStore );
        }

        public void DeletePaymentOptionStore( string storeID )
        {
            _paymentOptionStoreAccess.Delete( storeID );
        }

        public void DeletePaymentOptionStoreByName( string name )
        {
            _paymentOptionStoreAccess.DeleteByName( name );
        }

        public IList<PaymentOptionStore> GetAllPaymentOptionStore()
        {
            return _paymentOptionStoreAccess.GetAll();
        }

        public IList<PaymentOptionStore> GetAllPaymentOptionStoreByName(string name )
        {
            return _paymentOptionStoreAccess.GetAllByName( name );
        }

        #endregion
    }
}
