using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Domain.Tax;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class TaxClassRepository : ITaxClassRepository
    {
        #region Private

        private const string RepositoryName = "TaxClassGroupRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetTaxClassRuleByTaxClassIDSignature = "GetTaxClassRuleByTaxClassID";
        private const string GetOneTaxClassRuleSignature = "GetOneTaxClassRule";
        private const string GetAllTaxClassRuleSignature = "GetAllTaxClassRule";
        private const string SearchTaxClassRuleSignature = "SearchTaxClassRule";
        private const string GetTaxClassRuleTableSchemaSignature = "GetTaxClassRuleTableSchema";

        private const string GetAllSignature = "GetAll";
        private const string SearchTaxClassSignature = "SearchTaxClass";
        private const string GetTableSchemaSignature = "GetTableSchema";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "TaxClass", "TaxClassRule" } );

        private TaxClassRepositoryNoCache _repository = new TaxClassRepositoryNoCache();

        #endregion

        #region Constructors

        public TaxClassRepository()
        {
        }

        public TaxClassRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion


        // Functions taht are intended to be used internally inside aggregates.
        #region Aggregate Methods

        public IList<TaxClassRule> GetTaxClassRulesByTaxClassID( string taxClassID )
        {
            IList<TaxClassRule> list =
                (IList<TaxClassRule>) _dataCache.GetValue(
                GetTaxClassRuleByTaxClassIDSignature,
                _cacheDependency,
                taxClassID );

            if (list == null)
            {
                list = _repository.GetTaxClassRulesByTaxClassID( taxClassID );
                _dataCache.SetValue( GetTaxClassRuleByTaxClassIDSignature, list, taxClassID );
            }

            return list;
        }

        public TaxClassRule GetOneTaxClassRule( string taxClassRuleID )
        {
            TaxClassRule result = (TaxClassRule) _dataCache.GetValue( 
                GetOneTaxClassRuleSignature, 
                _cacheDependency, 
                taxClassRuleID );

            if (result == null)
            {
                result = _repository.GetOneTaxClassRule( taxClassRuleID );
                _dataCache.SetValue( GetOneTaxClassRuleSignature, result, taxClassRuleID );
            }

            return result;
        }

        public IList<TaxClassRule> GetAllTaxClassRules( string sortBy )
        {
            Pair result = (Pair) _dataCache.GetValue( GetAllTaxClassRuleSignature, _cacheDependency, sortBy );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.GetAllTaxClassRules( sortBy );
                _dataCache.SetValue( GetAllTaxClassRuleSignature, result, sortBy );
            }

            return (IList<TaxClassRule>) result.First;
        }

        public IList<TaxClassRule> SearchTaxClassRules(
            string taxClassID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchTaxClassRuleSignature,
                _cacheDependency,
                taxClassID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchTaxClassRules(
                    taxClassID, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchTaxClassRuleSignature,
                    result,
                    taxClassID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<TaxClassRule>) result.First;
        }

        public IList<TableSchemaItem> GetTaxClassRuleTableSchemas()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                GetTaxClassRuleTableSchemaSignature, _cacheDependency
            );
            if (list == null)
            {
                list = _repository.GetTaxClassRuleTableSchemas();
                _dataCache.SetValue(
                    GetTaxClassRuleTableSchemaSignature,
                    list );
            }

            return list;
        }

        #endregion

        #region Public Methods

        public TaxClass Save( TaxClass taxClass )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            TaxClass newTaxClass = _repository.Save( taxClass );
            _dataCache.ClearRepositoryCache();
            return newTaxClass;
        }

        public TaxClass GetOne( string taxClassID )
        {
            TaxClass result = (TaxClass) _dataCache.GetValue( GetOneSignature, _cacheDependency, taxClassID );

            if (result == null)
            {
                result = _repository.GetOne( taxClassID );
                _dataCache.SetValue( GetOneSignature, result, taxClassID );
            }

            return result;
        }

        public void Delete( string taxClassID )
        {
            _repository.Delete( taxClassID );      
            _dataCache.ClearRepositoryCache();
        }

        public IList<TaxClass> GetAll( string sortBy )
        {
            IList<TaxClass> result = (IList<TaxClass>) _dataCache.GetValue( GetAllSignature, _cacheDependency, sortBy );

            if (result == null)
            {
                result = _repository.GetAll( sortBy );
                _dataCache.SetValue( GetAllSignature, result, sortBy );
            }

            return result;
        }

        public IList<TaxClass> SearchTaxClasses(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchTaxClassSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
            );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchTaxClasses(
                    sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchTaxClassSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<TaxClass>) result.First;
        }

        public IList<TableSchemaItem> GetTableSchemas()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                GetTableSchemaSignature, _cacheDependency
                );

            if (list == null)
            {
                list = _repository.GetTableSchemas();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                );
            }

            return list;
        }

        #endregion
    }
}
