using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.Base;
using Vevo.Domain.Configurations;
using Vevo.Domain.Exceptions;
using Vevo.Domain.Users;
using Vevo.Domain.Discounts;
using Vevo.Base.Domain;

namespace Vevo.Domain.Shipping
{
    /// <summary>
    /// This class support for table Shipping and ShippingLocale
    /// </summary>
    [Serializable]
    public class ShippingOption : ILocaleLoader, ICloneable
    {
        #region Enum Type

        public enum ShippingZoneAllowedType
        {
            Unknown,
            Worldwide,
            Allow,
            Deny
        }
        #endregion


        #region Private

        private string _shippingID = "0";
        private string _shippingOptionTypeID = "0";
        private bool _freeShipping = false;
        private decimal _freeShippingValue = 0;
        private decimal _fixedShippingCost = 0;
        private decimal _shippingCostFirstItem = 0;
        private decimal _shippingCostNextItem = 0;
        private bool _isEnabled = true;
        private decimal _handlingFee = 0;

        private Culture _defaultCulture;
        private LocaleCollection<ShippingOptionLocale> _locales;

        private ShippingOptionType _shippingOptionType;
        private IList<ShippingZone> _shippingZone;
        private ShippingZoneAllowedType _allowedType = ShippingZoneAllowedType.Worldwide;

        private object CreateObject( string typeName, params object[] constructorParameters )
        {
            Type type = Type.GetType( typeName );
            if (type == null)
                throw new DomainException( "Invalid Type Name: " + typeName );

            return Activator.CreateInstance( type, constructorParameters );
        }

        private bool IsAddressMatchShipppingZone( ShippingAddress shippingAddress )
        {
            foreach (ShippingZone shippingZone in ShippingZone)
            {
                ShippingZoneGroup zone = DataAccessContext.ShippingZoneGroupRepository.GetOne( shippingZone.ZoneGroupID );
                if (zone.IsAddressMatch( shippingAddress ))
                    return true;
            }

            return false;
        }
        #endregion


        #region Protected Constructors

        protected ShippingOption( Culture defaultCulture, bool isNullInstance )
            : this( defaultCulture )
        {
            if (isNullInstance)
            {
                _shippingZone = new List<ShippingZone>();
            }
        }

        #endregion


        #region Public Constructors

        public ShippingOption( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<ShippingOptionLocale>( this );
        }

        #endregion


        #region Public Properties

        public static ShippingOption Null
        {
            get { return NullShippingOption.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string ShippingID { get { return _shippingID; } set { _shippingID = value; } }
        public string ShippingOptionTypeID { get { return _shippingOptionTypeID; } set { _shippingOptionTypeID = value; } }
        public bool FreeShipping { get { return _freeShipping; } set { _freeShipping = value; } }
        public decimal FreeShippingValue { get { return _freeShippingValue; } set { _freeShippingValue = value; } }
        public decimal FixedShippingCost { get { return _fixedShippingCost; } set { _fixedShippingCost = value; } }
        public decimal ShippingCostFirstItem { get { return _shippingCostFirstItem; } set { _shippingCostFirstItem = value; } }
        public decimal ShippingCostNextItem { get { return _shippingCostNextItem; } set { _shippingCostNextItem = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }
        public decimal HandlingFee { get { return _handlingFee; } set { _handlingFee = value; } }
        public ShippingZoneAllowedType AllowedType { get { return _allowedType; } set { _allowedType = value; } }

        public IList<ShippingZone> ShippingZone
        {
            get
            {
                if (_shippingZone == null)
                    _shippingZone = new EntityList<ShippingZone>(
                        DataAccessContext.ShippingOptionRepository.GetShippingZone( ShippingID ) );

                return _shippingZone;
            }
        }

        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<ShippingOptionLocale> Locales { get { return _locales; } }


        //------------------------------------------------------------
        // Locale items
        public string ShippingName
        {
            get { return Locales[DefaultCulture].ShippingName; }
            set { Locales.GetNoLoad( DefaultCulture ).ShippingName = value; }
        }
        // End Locale items
        //------------------------------------------------------------

        #endregion


        #region Custom Properties

        public ShippingOptionType ShippingOptionType
        {
            get
            {
                if (_shippingOptionType == null)
                    _shippingOptionType =
                        DataAccessContext.ShippingOptionTypeRepository.GetOne( ShippingOptionTypeID );

                return _shippingOptionType;
            }
        }

        #endregion


        #region ILocaleLoader Members

        public IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (ShippingOptionLocale locale in DataAccessContext.ShippingOptionRepository.GetLocales( ShippingID ))
                result.Add( locale );

            return result;
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            ShippingOption newObject = (ShippingOption) MemberwiseClone();
            newObject._locales = (LocaleCollection<ShippingOptionLocale>) Locales.Clone();
            return newObject;
        }

        #endregion


        #region Public Methods

        public bool CanShipTo( ShippingAddress shippingAddress )
        {
            switch (AllowedType)
            {
                case ShippingZoneAllowedType.Allow:
                    {
                        return IsAddressMatchShipppingZone( shippingAddress );
                    }
                case ShippingZoneAllowedType.Deny:
                    {
                        return !IsAddressMatchShipppingZone( shippingAddress );
                    }
                default:
                    {
                        return true;
                    }
            }
        }

        // For real-time shipping gateways (e.g. FedEx, UPS), this method would take long time
        // since it requests list of available shipping choices using HTTP post or web services.

        public IList<ShippingChoice> RequestRealTimeShippingChoices(
            ShippingAddress shippingAddress,
            Orders.Cart shoppingCart,
            decimal subtotal,
            bool isContainFreeShippingItem,
            Coupon coupon,
            out string errorMessage,
            out string restriction )
        {
            IShippingGateway gateway = (IShippingGateway) CreateObject(
                ShippingOptionType.RealTimeGatewayClassName,
                this );

            return gateway.RequestShippingChoices(
                shippingAddress,
                shoppingCart,
                subtotal,
                isContainFreeShippingItem,
                coupon,
                out errorMessage,
                out restriction );
        }

        public ShippingMethod CreateNonRealTimeShippingMethod()
        {
            return (ShippingMethod) CreateObject(
                ShippingOptionType.ProviderClassName,
                this,
                null );
        }

        public virtual ShippingMethod CreateShippingMethod(
            ShippingChoice shippingChoice )
        {
            return (ShippingMethod) CreateObject(
                ShippingOptionType.ProviderClassName,
                this,
                shippingChoice );
        }

        public decimal GetHandlingFee( bool isHandlingFeeEnabled, decimal shippingCost )
        {
            if (!isHandlingFeeEnabled
                || shippingCost == 0)
                return 0;
            else
                return HandlingFee;
        }

        #endregion

    }
}
